package net.cattaka.rdbassistant.gui.table;

import java.awt.Font;
import javax.swing.JTable;
import javax.swing.JViewport;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import net.cattaka.rdbassistant.gui.RdbaTextInterface;
import net.cattaka.swing.StdScrollPane;

public class ResultSetTablePanel extends StdScrollPane {
	private static final long serialVersionUID = 1L;
	private JTableForDisplay table;
	private JTableForDisplay rowHeader;
	private JViewport tableViewport;
	private JViewport rowHeaderViewport;

	class RowHeaderTableModel implements ResultSetTableModel {
		private ResultSetTableModel tableModel;
		private int columnCharacters;
		
		public RowHeaderTableModel(ResultSetTableModel tableModel) {
			this.tableModel = tableModel;
			this.columnCharacters = 1;
			int n = tableModel.getRowCount();
			for (int i=0;i<n;i++) {
				int l = tableModel.getRowName(i).toString().length();
				if (this.columnCharacters < l) {
					this.columnCharacters = l;
				}
			}
		}
		
		public void addTableModelListener(TableModelListener l) {
			return;
		}

		public Class<?> getColumnClass(int columnIndex) {
			return tableModel.getRowHeaderClass();
		}

		public int getColumnCount() {
			return 1;
		}

		public String getColumnName(int columnIndex) {
			return "";
		}

		public int getRowCount() {
			return tableModel.getRowCount();
		}

		public Object getValueAt(int rowIndex, int columnIndex) {
			return tableModel.getRowName(rowIndex);
		}

		public boolean isCellEditable(int rowIndex, int columnIndex) {
			return false;
		}

		public void removeTableModelListener(TableModelListener l) {
			return;
		}

		public void setValueAt(Object value, int rowIndex, int columnIndex) {
			return;
		}

		public int[] getColumnCharacters() {
			return new int []{this.columnCharacters};
		}

		public String getRowName(int row) {
			return "";
		}

		public String getTableName() {
			return tableModel.getTableName();
		}

		public Class<?> getRowHeaderClass() {
			return tableModel.getRowHeaderClass();
		}
		
	}
	
	public ResultSetTablePanel() {
		super();
		this.table = new JTableForDisplay();
		this.tableViewport = new JViewport();
		{
			tableViewport.setView(table);
			this.setViewport(tableViewport);
		}
		
		this.rowHeader = new JTableForDisplay();
		this.rowHeaderViewport = new JViewport();
		{
			this.rowHeader.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
			this.rowHeader.setForeground(this.table.getTableHeader().getForeground());
			this.rowHeader.setBackground(this.table.getTableHeader().getBackground());
			rowHeaderViewport.setView(rowHeader);
			this.setRowHeader(rowHeaderViewport);
			
			this.setRowHeader(rowHeaderViewport);
		}
	}

	
	public int getAutoResizeMode() {
		return table.getAutoResizeMode();
	}


	public boolean getCellSelectionEnabled() {
		return table.getCellSelectionEnabled();
	}


	public void setAutoResizeMode(int mode) {
		table.setAutoResizeMode(mode);
	}


	public void setCellSelectionEnabled(boolean cellSelectionEnabled) {
		table.setCellSelectionEnabled(cellSelectionEnabled);
	}


	public TableModel getModel() {
		return table.getModel();
	}

//	public void setModel(TableModel dataModel) {
//		table.setModel(dataModel);
//	}

	public String getFilteredString(int rowIndex) {
		Object value = table.getValueAt(rowIndex, 0);
		return value.toString();
	}

	@Override
	public void setFont(Font font) {
		super.setFont(font);
		if (table != null) {
			table.setFont(font);
		}
		if (rowHeader != null) {
			rowHeader.setFont(font);
		}
	}


	public void setResultSetTableModel(ResultSetTableModel dataModel) {
		table.setResultSetTableModel(dataModel);
		this.rowHeader.setResultSetTableModel(new RowHeaderTableModel(dataModel));
		this.rowHeaderViewport.setPreferredSize(this.rowHeader.getPreferredSize());
	}

	public RdbaTextInterface getRdbaTextInterface() {
		return table.getRdbaTextInterface();
	}

	public void setRdbaTextInterface(RdbaTextInterface rdbaTextInterface) {
		table.setRdbaTextInterface(rdbaTextInterface);
	}
	
	public boolean isTransposed() {
		return (table.getModel() instanceof TransposedTableModel);
	}
	/**
	 * 現在のモデルがResultSetTableModelでない場合はエラーを吐くので注意。
	 * @param transposed
	 */
	public void setTransposed(boolean transposed) {
		if (isTransposed()) {
			if (!transposed) {
				TransposedTableModel tm = (TransposedTableModel)table.getModel();
				this.setResultSetTableModel(tm.getTableModel());
			}
		} else {
			if (transposed) {
				if (table.getModel() instanceof ResultSetTableModel) {
					TransposedTableModel tm = new TransposedTableModel((ResultSetTableModel)table.getModel());
					this.setResultSetTableModel(tm);
				}
			}
		}
	}
}
